@@ -107,8 +107,8 @@ printex(model, ParameterAs = model@param.names, printInit=TRUE, printRS=TRUE,
 #tools::texi2pdf("RSNonlinearODE_1.tex")
 #system(paste(getOption("pdfviewer"), "RSNonlinearODE_1.pdf"))
 
-model$ub[model@param.names%in%c("int_1", "int_2", "slp_1", "slp_2")]<-c(0,0,10,10)
-model$lb[model@param.names%in%c("int_1", "int_2", "slp_1", "slp_2")]<-c(-10,-10,0,0)
+model$ub[c("int_1", "int_2", "slp_1", "slp_2")] <- c(0,0,10,10)
+model$lb[c("int_1", "int_2", "slp_1", "slp_2")] <- c(-10,-10,0,0)
 # Estimate free parameters
 res <- dynr.cook(model)
 
